==> soa-5.0.0-src.zip: soa-5.0.0-src/soa-work/esb/checkout/product/services/jbpm/src/main/java/org/jboss/soa/esb/services/jbpm/integration/msg/JmsMessageServiceFactory.java <==
/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.soa.esb.services.jbpm.integration.msg;

import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.jbpm.JbpmException;
import org.jbpm.svc.Service;
import org.jbpm.svc.ServiceFactory;

/**
 * Based on the jBPM JmsMessageServiceFactory, this service delivers messages to the
 * job/command listeners over JCA/JMS inflow.
 * 
 * <h3>Configuration</h3>
 * 
 * The JMS message service factory exposes the following configurable fields.
 * 
 * <ul>
 * <li><code>connectionFactoryJndiName</code></li>
 * <li><code>destinationJndiName</code></li>
 * <li><code>commandJndiName</code></li>
 * <li><code>dlqJndiName</code></li>
 * <li><code>isCommitEnabled</code></li>
 * <li><code>idleInterval</code></li>
 * <li><code>maxIdleInterval</code></li>
 * <li><code>maxRetryJobs</code></li>
 * </ul>
 * 
 * @author Tom Baeyens
 * @author Alejandro Guizar
 */
public final class JmsMessageServiceFactory implements ServiceFactory
{
  private static final long serialVersionUID = 1L;

  /**
   * The JNDI name for locating the JMS connection factory.
   */
  String connectionFactoryJndiName = "java:/JmsXA";
  /**
   * The JNDI name for locating the jBPM job queue destination.
   */
  String destinationJndiName = "queue/JbpmJobQueue";
  /**
   * The JNDI name for locating the jBPM command queue destination.
   */
  String commandJndiName = "queue/JbpmCommandQueue";
  /**
   * The JNDI name for locating the jBPM dlq destination.
   */
  String dlqJndiName = "queue/JbpmDLQ";
  /**
   * A flag indicating whether commit should be invoked on the JMS session.
   */
  boolean isCommitEnabled = false;
  /**
   * The default idle interval for the retry executor.
   */
  int idleInterval = 15000; // 15 secs
  /**
   * The maximum idle interval for the retry executor.
   */
  int maxIdleInterval = 3600000; // 1 hour
  /**
   * The maximum number of jobs to reschedule on each poll.
   */
  int maxRetryJobs = 20;

  /**
   * The JMS connection factory associated with the scheduler.
   */
  private ConnectionFactory connectionFactory;
  /**
   * The JMS destination for the jBPM job queue.
   */
  private Destination destination;
  /**
   * The JMS destination for the jBPM command queue.
   */
  private Destination commandDestination;
  /**
   * The JMS destination for the jBPM dlq.
   */
  private Destination dlqDestination;
  /**
   * The retry executor.
   */
  private RetryExecutor retryExecutor ;
  /**
   * The thread associated with the retry executor.
   */
  private Thread retryExecutorThread ;

  /**
   * Construct the JMS message service factory.
   */
  public JmsMessageServiceFactory()
  {
    retryExecutor = new RetryExecutor(idleInterval, maxIdleInterval, maxRetryJobs) ;
    retryExecutorThread = new Thread(retryExecutor) ;
    retryExecutorThread.start() ;
  }
  
  /**
   * Get the JMS connection factory associated with this scheduler.
   * @return The associated JMS connection factory.
   */
  public ConnectionFactory getConnectionFactory()
  {
    if (connectionFactory == null)
    {
      try
      {
        connectionFactory = (ConnectionFactory)lookup(connectionFactoryJndiName);
      }
      catch (NamingException e)
      {
        throw new JbpmException("could not retrieve message connection factory", e);
      }
    }
    return connectionFactory;
  }

  /**
   * Get the jBPM job queue destination associated with this message service.
   * @return The associated jBPM job queue.
   */
  public Destination getDestination()
  {
    if (destination == null)
    {
      try
      {
        destination = (Destination)lookup(destinationJndiName);
      }
      catch (NamingException e)
      {
        throw new JbpmException("could not retrieve job destination", e);
      }
    }
    return destination;
  }

  /**
   * Get the jBPM command queue destination associated with this message service.
   * @return The associated jBPM command queue.
   */
  public Destination getCommandDestination()
  {
    if (commandDestination == null)
    {
      try
      {
        commandDestination = (Destination)lookup(commandJndiName);
      }
      catch (NamingException e)
      {
        throw new JbpmException("could not retrieve command destination", e);
      }
    }
    return commandDestination;
  }

  /**
   * Get the jBPM dlq destination associated with this message service.
   * @return The associated jBPM dlq.
   */
  public Destination getDLQDestination()
  {
    if (dlqDestination == null)
    {
      try
      {
          dlqDestination = (Destination)lookup(dlqJndiName);
      }
      catch (NamingException e)
      {
        throw new JbpmException("could not retrieve dlq destination", e);
      }
    }
    return dlqDestination;
  }

  /**
   * Get the commitEnabled flag.
   * @return true if commit is to be invoked on the JMS session, false otherwise.
   */
  public boolean isCommitEnabled()
  {
    return isCommitEnabled;
  }

  /**
   * Lookup a name within JNDI.
   * @param name The JNDI name to locate.
   * @return The object bound to the name.
   * @throws NamingException For errors locating the specified object within JNDI.
   */
  private static Object lookup(String name) throws NamingException
  {
    Context initial = new InitialContext();
    try
    {
      return initial.lookup(name);
    }
    finally
    {
      initial.close();
    }
  }

  /**
   * Create a JMS message service based on this factory.
   */
  public Service openService()
  {
    try
    {
      return new JmsMessageService(getConnectionFactory(), getDestination(), isCommitEnabled);
    }
    catch (JMSException e)
    {
      throw new JbpmException("couldn't open message session", e);
    }
  }

  /**
   * Close this factory.
   */
  public void close()
  {
    connectionFactory = null;
    destination = null;
    commandDestination = null;
    dlqDestination = null;
    try
    {
        retryExecutor.terminate() ;
        try
        {
            retryExecutorThread.join() ;
        }
        catch (final InterruptedException ie) {}
    }
    finally
    {
        retryExecutorThread = null ;
    }
  }
}

==> soa-5.0.2-src.zip: soa-5.0.2-src/soa-work/esb/checkout/product/services/jbpm/src/main/java/org/jboss/soa/esb/services/jbpm/integration/msg/JmsMessageServiceFactory.java <==
/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.soa.esb.services.jbpm.integration.msg;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.naming.NamingException;

import org.jboss.soa.esb.services.jbpm.integration.AbstractJmsServiceFactory;
import org.jbpm.JbpmException;
import org.jbpm.svc.Service;

/**
 * Based on the jBPM JmsMessageServiceFactory, this service delivers messages to the
 * job/command listeners over JCA/JMS inflow.
 * 
 * <h3>Configuration</h3>
 * 
 * The JMS message service factory exposes the following configurable fields.
 * 
 * <ul>
 * <li><code>connectionFactoryJndiName</code></li>
 * <li><code>destinationJndiName</code></li>
 * <li><code>commandJndiName</code></li>
 * <li><code>dlqJndiName</code></li>
 * <li><code>isCommitEnabled</code></li>
 * <li><code>idleInterval</code></li>
 * <li><code>maxIdleInterval</code></li>
 * <li><code>maxRetryJobs</code></li>
 * <li><code>providerAdapterJNDI</code></li>
 * </ul>
 * 
 * @author Tom Baeyens
 * @author Alejandro Guizar
 */
public final class JmsMessageServiceFactory extends AbstractJmsServiceFactory
{
  private static final long serialVersionUID = 1L;

  /**
   * The JNDI name for locating the jBPM job queue destination.
   */
  String destinationJndiName = "queue/JbpmJobQueue";
  /**
   * The JNDI name for locating the jBPM command queue destination.
   */
  String commandJndiName = "queue/JbpmCommandQueue";
  /**
   * The JNDI name for locating the jBPM dlq destination.
   */
  String dlqJndiName = "queue/JbpmDLQ";
  /**
   * The default idle interval for the retry executor.
   */
  int idleInterval = 15000; // 15 secs
  /**
   * The maximum idle interval for the retry executor.
   */
  int maxIdleInterval = 3600000; // 1 hour
  /**
   * The maximum number of jobs to reschedule on each poll.
   */
  int maxRetryJobs = 20;

  /**
   * The JMS destination for the jBPM job queue.
   */
  private Destination destination;
  /**
   * The JMS destination for the jBPM command queue.
   */
  private Destination commandDestination;
  /**
   * The JMS destination for the jBPM dlq.
   */
  private Destination dlqDestination;
  /**
   * The retry executor.
   */
  private RetryExecutor retryExecutor ;
  /**
   * The thread associated with the retry executor.
   */
  private Thread retryExecutorThread ;

  /**
   * Construct the JMS message service factory.
   */
  public JmsMessageServiceFactory()
  {
    retryExecutor = new RetryExecutor(idleInterval, maxIdleInterval, maxRetryJobs) ;
    retryExecutorThread = new Thread(retryExecutor) ;
    retryExecutorThread.start() ;
  }

  /**
   * Get the jBPM job queue destination associated with this message service.
   * @return The associated jBPM job queue.
   */
  public Destination getDestination()
  {
    if (destination == null)
    {
      try
      {
        destination = (Destination)lookup(destinationJndiName);
      }
      catch (NamingException e)
      {
        throw new JbpmException("could not retrieve job destination", e);
      }
    }
    return destination;
  }

  /**
   * Get the jBPM command queue destination associated with this message service.
   * @return The associated jBPM command queue.
   */
  public Destination getCommandDestination()
  {
    if (commandDestination == null)
    {
      try
      {
        commandDestination = (Destination)lookup(commandJndiName);
      }
      catch (NamingException e)
      {
        throw new JbpmException("could not retrieve command destination", e);
      }
    }
    return commandDestination;
  }

  /**
   * Get the jBPM dlq destination associated with this message service.
   * @return The associated jBPM dlq.
   */
  public Destination getDLQDestination()
  {
    if (dlqDestination == null)
    {
      try
      {
          dlqDestination = (Destination)lookup(dlqJndiName);
      }
      catch (NamingException e)
      {
        throw new JbpmException("could not retrieve dlq destination", e);
      }
    }
    return dlqDestination;
  }

  /**
   * Create a JMS message service based on this factory.
   */
  public Service openService()
  {
    try
    {
      return new JmsMessageService(getConnectionFactory(), getDestination(), isCommitEnabled());
    }
    catch (JMSException e)
    {
      throw new JbpmException("couldn't open message session", e);
    }
  }

  /**
   * Close this factory.
   */
  public void close()
  {
    super.close() ;
    destination = null;
    commandDestination = null;
    dlqDestination = null;
    try
    {
        retryExecutor.terminate() ;
        try
        {
            retryExecutorThread.join() ;
        }
        catch (final InterruptedException ie) {}
    }
    finally
    {
        retryExecutorThread = null ;
    }
  }
}
